/* ************************************************************************** */
/*                                                                            */
/*                                                        :::      ::::::::   */
/*   cmd_cd.c                                           :+:      :+:    :+:   */
/*                                                    +:+ +:+         +:+     */
/*   By: tmaze <tmaze@student.42.fr>                +#+  +:+       +#+        */
/*                                                +#+#+#+#+#+   +#+           */
/*   Created: 2019/01/07 16:44:40 by tmaze             #+#    #+#             */
/*   Updated: 2019/01/27 16:32:01 by tmaze            ###   ########.fr       */
/*                                                                            */
/* ************************************************************************** */

#include "minishell.h"

void	put_error_cd(char *file, char *msg)
{
	ft_putstr("cd: ");
	ft_putstr(file);
	ft_putstr(": ");
	ft_putendl(msg);
}

char	*check_path_slash_cd(char *exec)
{
	int			i;
	struct stat	info;

	if (ft_isin(exec, '/') != -1 && ft_isin(exec, '.') == -1)
	{
		if ((i = access(exec, F_OK)) != 0)
			put_error_cd(exec, "no such file or directory");
		if (i != 0)
			return (NULL);
		if ((i = stat(exec, &info)) != 0)
			put_error_cd(exec, "can't determine info");
		if (i != 0)
			return (NULL);
		if (!S_ISDIR(info.st_mode))
		{
			put_error_cd(exec, "not a directory");
			return (NULL);
		}
		if ((i = access(exec, X_OK)) != 0)
			put_error_cd(exec, "permission denied");
		if (i != 0)
			return (NULL);
		return (exec);
	}
	return (NULL);
}

int		cmd_cd_core(char *path, t_list **env)
{
	t_envelem	*pwd;
	char		*oldpwd;

	if (check_path_slash_cd(path) == NULL)
	{
		put_error_cd(path, "invalid path");
		return (1);
	}
	if (chdir(path) == -1
		|| (pwd = env_getelemfromkey("PWD", *env)) == NULL)
	{
		put_error_cd(path, "error");
		return (1);
	}
	if ((oldpwd = ft_strdup(pwd->val)) == NULL)
	{
		put_error_cd(path, "error");
		return (1);
	}
	if (env_addupdate("PWD", path, env) == NULL
		|| env_addupdate("OLDPWD", oldpwd, env) == NULL)
	{
		put_error_cd(path, "error");
		ft_strdel(&oldpwd);
		return (1);
	}
	ft_strdel(&oldpwd);
	return (0);
}

char	cd_getparams(char **argv, size_t *i)
{
	char	ret;
	size_t	j;

	ret = 'L';
	while (argv[++*i] && argv[*i][0] == '-' && ft_strlen(argv[*i]) > 1)
	{
		j = 0;
		while (argv[*i][j] && (argv[*i][j] == 'L' || argv[*i][j] == 'P'))
			ret = argv[*i][j++];
		if (argv[*i][j] && argv[*i][j] != 'L' && argv[*i][j] != 'P')
		{
			ret = argv[*i][j];
			put_error_cd(&ret, "invalid option");
			put_error_cd("usage", "cd [-L|-P] [dir]");
			ret = '\0';
			break ;
		}
	}
	return (ret);
}

int		ft_isfstcmp(char *path, char *cmp)
{
	size_t	i;

	i = 0;
	while (path[i] == '/')
		i++;
	return (ft_strncmp(&path[i], cmp, ft_strlen(cmp)));
}

int		cmd_cd2(char **argv, t_list **env)
{
	t_envelem	*elem;
	char		*path;
	char		*tmp;
	char		opt;
	size_t		i;

	i = 0;
	path = NULL;
	if ((opt = cd_getparams(argv, &i)) == '\0')
		return (1);
	if (!argv[i] && (elem = env_getelemfromkey("HOME", *env)) != NULL && elem->val != NULL)
		tmp = elem->val;
	else if (argv[i] && argv[i][0] == '.' && (elem = env_getelemfromkey("PWD", *env)) != NULL && elem->val != NULL)
	{
		if ((tmp = ft_strnew(ft_strlen(argv[i]) + ft_strlen(elem->val) + 1)) == NULL)
			put_error_cd(argv[i], "memory error");
		if (tmp == NULL)
			return (1);
		ft_strcat(tmp, elem->val);
		ft_strcat(tmp, "/");
		ft_strcat(tmp, argv[i]);
	}
	else if ()
	return (0);
}

int		cmd_cd(char **argv, t_list **env)
{
	t_envelem	*elem;
	size_t		i;
	char		*path;
	char		*tmp;
	char		opt;
	int			ret;

	i = 0;
	if ((opt = cd_getparams(argv, &i)) == '\0')
		return (1);
	if (!argv[i] && (elem = env_getelemfromkey("HOME", *env)) != NULL && elem->val != NULL)
		return (cmd_cd_core(elem->val, env));
	else if (argv[i] && argv[1][0] == '/')
		return (cmd_cd_core(argv[1], env));
	else if (argv[i] && argv[i][0] == '-' && (elem = env_getelemfromkey("OLDPWD", *env)) != NULL)
		return (cmd_cd_core(elem->val, env));
	else if (argv[i] && argv[i][0] == '.' && (elem = env_getelemfromkey("PWD", *env)) != NULL && elem->val != NULL)
	{
		if ((tmp = ft_strnew(ft_strlen(elem->val) + ft_strlen(argv[i]) + 1)) == NULL)
			put_error_cd(argv[1], "memory error");
		if (tmp == NULL)
			return (1);
		ft_strcpy(tmp, elem->val);
		tmp[ft_strlen(tmp)] = '/';
		ft_strcat(tmp, argv[1]);
		if (ft_realpath(tmp, &path) == NULL)
			put_error_cd(argv[1], "memory error");
		ret = cmd_cd_core(path, env);
		ft_strdel(&tmp);
		ft_strdel(&path);
		return (ret);
	}
	else if (argv[1] && (elem = env_getelemfromkey("PWD", *env)) != NULL && elem->val != NULL)
	{
		if ((path = ft_strnew(ft_strlen(elem->val) + ft_strlen(argv[1]) + 1)) == NULL)
			put_error_cd(argv[1], "memory error");
		if (path == NULL)
			return (1);
		ft_strcpy(path, elem->val);
		path[ft_strlen(path)] = '/';
		ft_strcat(path, argv[1]);
		ft_putendl(path);
		ft_putendl("plop");
		ret = cmd_cd_core(path, env);
		ft_strdel(&tmp);
		ft_strdel(&path);
		return (ret);
	}
	return (1);
}
